@@ -1,10 +1,10 @@
 # encoding=utf-8
 '''Application support.'''
+from http.cookiejar import CookieJar
 import atexit
 import codecs
 import functools
 import gettext
-from http.cookiejar import CookieJar
 import itertools
 import logging
 import os.path
@@ -27,6 +27,7 @@ from wpull.debug import DebugConsoleHandler
 from wpull.dns import Resolver
 from wpull.engine import Engine
 from wpull.factory import Factory
+from wpull.ftp.client import Client as FTPClient
 from wpull.hook import HookEnvironment
 from wpull.http.client import Client as HTTPClient
 from wpull.http.proxy import ProxyAdapter
@@ -36,9 +37,12 @@ from wpull.http.robots import RobotsTxtChecker
 from wpull.http.stream import Stream as HTTPStream
 from wpull.http.web import WebClient
 from wpull.namevalue import NameValueRecord
-from wpull.phantomjs import PhantomJSClient
-from wpull.processor.phantomjs import PhantomJSController
-from wpull.processor.rule import FetchRule, ResultRule
+from wpull.driver.phantomjs import PhantomJSClient
+from wpull.processor.delegate import DelegateProcessor
+from wpull.processor.ftp import FTPProcessor, FTPProcessorFetchParams, \
+    FTPProcessorInstances
+from wpull.processor.rule import FetchRule, ResultRule, ProcessingRule
+from wpull.coprocessor.phantomjs import PhantomJSController
 from wpull.processor.web import WebProcessor, WebProcessorFetchParams, \
     WebProcessorInstances
 from wpull.proxy import HTTPProxyServer
@@ -55,18 +59,19 @@ from wpull.scraper.javascript import JavaScriptScraper
 from wpull.scraper.sitemap import SitemapScraper
 from wpull.stats import Statistics
 from wpull.url import URLInfo
-from wpull.urlfilter import (DemuxURLFilter, HTTPSOnlyFilter, HTTPFilter,
+from wpull.urlfilter import (DemuxURLFilter, HTTPSOnlyFilter, SchemeFilter,
                              BackwardDomainFilter, HostnameFilter, TriesFilter,
                              RecursiveFilter, LevelFilter,
                              SpanHostsFilter, RegexFilter, DirectoryFilter,
-                             BackwardFilenameFilter, ParentFilter)
+                             BackwardFilenameFilter, ParentFilter,
+                             FollowFTPFilter)
 from wpull.util import ASCIIStreamWriter
-import wpull.version
 from wpull.waiter import LinearWaiter
 from wpull.wrapper import CookieJarWrapper
 from wpull.writer import (PathNamer, NullWriter, OverwriteFileWriter,
                           IgnoreFileWriter, TimestampingFileWriter,
                           AntiClobberFileWriter)
+import wpull.version
 
 
 _logger = logging.getLogger(__name__)
@@ -98,9 +103,14 @@ class Builder(object):
             'DemuxDocumentScraper': DemuxDocumentScraper,
             'DemuxRecorder': DemuxRecorder,
             'DemuxURLFilter': DemuxURLFilter,
+            'FTPProcessor': FTPProcessor,
             'Engine': Engine,
             'ElementWalker': ElementWalker,
             'FetchRule': FetchRule,
+            'FileWriter': NullWriter,
+            'FTPClient': FTPClient,
+            'FTPProcessorFetchParams': FTPProcessorFetchParams,
+            'FTPProcessorInstances': FTPProcessorInstances,
             'HTTPProxyServer': HTTPProxyServer,
             'HTMLParser': NotImplemented,
             'HTMLScraper': HTMLScraper,
@@ -110,6 +120,8 @@ class Builder(object):
             'PhantomJSClient': PhantomJSClient,
             'PhantomJSController': PhantomJSController,
             'PrintServerResponseRecorder': PrintServerResponseRecorder,
+            'ProcessingRule': ProcessingRule,
+            'Processor': DelegateProcessor,
             'ProxyAdapter': ProxyAdapter,
             'ProgressRecorder': ProgressRecorder,
             'RedirectTracker': RedirectTracker,
@@ -391,7 +403,6 @@ class Builder(object):
 
             url_string_iter = itertools.chain(url_string_iter, urls)
 
-        sitemap_url_infos = set()
         base_url = self._args.base
 
         for url_string in url_string_iter:
@@ -406,23 +417,6 @@ class Builder(object):
             _logger.debug(__('Parsed URL {0}', url_info))
             yield url_info
 
-            if self._args.sitemaps:
-                sitemap_url_infos.update((
-                    URLInfo.parse(
-                        '{0}://{1}/robots.txt'.format(
-                            url_info.scheme,
-                            url_info.hostname_with_port)
-                    ),
-                    URLInfo.parse(
-                        '{0}://{1}/sitemap.xml'.format(
-                            url_info.scheme,
-                            url_info.hostname_with_port)
-                    )
-                ))
-
-        for url_info in sitemap_url_infos:
-            yield url_info
-
     def _read_input_file_as_lines(self):
         '''Read lines from input file and return them.'''
         input_file = codecs.getreader(
@@ -456,7 +450,7 @@ class Builder(object):
         args = self._args
 
         filters = [
-            HTTPSOnlyFilter() if args.https_only else HTTPFilter(),
+            HTTPSOnlyFilter() if args.https_only else SchemeFilter(),
             RecursiveFilter(
                 enabled=args.recursive, page_requisites=args.page_requisites
             ),
@@ -466,6 +460,7 @@ class Builder(object):
                 page_requisites='page-requisites' in args.span_hosts_allow,
                 linked_pages='linked-pages' in args.span_hosts_allow,
             ),
+            FollowFTPFilter(follow=args.follow_ftp),
         ]
 
         if args.no_parent:
@@ -599,7 +594,7 @@ class Builder(object):
 
             if args.phantomjs:
                 software_string += ' PhantomJS/{0}'.format(
-                    wpull.phantomjs.get_version(exe_path=args.phantomjs_exe)
+                    wpull.driver.phantomjs.get_version(exe_path=args.phantomjs_exe)
                 )
 
             url_table = self._factory['URLTable'] if args.warc_dedup else None
@@ -702,6 +697,12 @@ class Builder(object):
         Returns:
             Processor: An instance of :class:`.processor.BaseProcessor`.
         '''
+        web_processor = self._build_web_processor()
+        ftp_processor = self._build_ftp_processor()
+        return self._factory.new('Processor', web_processor, ftp_processor)
+
+    def _build_web_processor(self):
+        '''Build WebProcessor.'''
         args = self._args
         url_filter = self._factory.new('DemuxURLFilter',
                                        self._build_url_filters())
@@ -714,24 +715,34 @@ class Builder(object):
         fetch_rule = self._factory.new(
             'FetchRule',
             url_filter=url_filter, robots_txt_checker=robots_txt_checker)
-        result_rule = self._factory.new(
-            'ResultRule',
-            retry_connrefused=args.retry_connrefused,
-            retry_dns_error=args.retry_dns_error,
-        )
 
         waiter = self._factory.new('Waiter',
                                    wait=args.wait,
                                    random_wait=args.random_wait,
                                    max_wait=args.waitretry)
 
+        result_rule = self._factory.new(
+            'ResultRule',
+            ssl_verification=args.check_certificate,
+            retry_connrefused=args.retry_connrefused,
+            retry_dns_error=args.retry_dns_error,
+            waiter=waiter,
+            statistics=self._factory['Statistics'],
+        )
+
+        processing_rule = self._factory.new(
+            'ProcessingRule',
+            fetch_rule,
+            document_scraper=document_scraper,
+            sitemaps=self._args.sitemaps,
+        )
+
         web_processor_instances = self._factory.new(
             'WebProcessorInstances',
             fetch_rule=fetch_rule,
             result_rule=result_rule,
-            document_scraper=document_scraper,
+            processing_rule=processing_rule,
             file_writer=file_writer,
-            waiter=waiter,
             statistics=self._factory['Statistics'],
             phantomjs_controller=phantomjs_controller,
         )
@@ -751,6 +762,40 @@ class Builder(object):
 
         return processor
 
+    def _build_ftp_processor(self):
+        '''Build FTPProcessor.'''
+        ftp_client = self._build_ftp_client()
+
+        fetch_params = self._factory.new(
+            'FTPProcessorFetchParams',
+            remove_listing=self._args.remove_listing
+        )
+
+        instances = self._factory.new(
+            'FTPProcessorInstances',
+            fetch_rule=self._factory['FetchRule'],
+            result_rule=self._factory['ResultRule'],
+            processing_rule=self._factory['ProcessingRule'],
+            file_writer=self._factory['FileWriter'],
+        )
+
+        return self._factory.new(
+            'FTPProcessor',
+            ftp_client,
+            self._args.directory_prefix,
+            fetch_params,
+            instances
+        )
+
+    def _build_ftp_client(self):
+        '''Build FTP client.'''
+        return self._factory.new(
+            'FTPClient',
+            connection_pool=self._factory['ConnectionPool'],
+            recorder=self._factory['DemuxRecorder'],
+            proxy_adapter=self._factory.instance_map.get('ProxyAdapter')
+            )
+
     def _build_file_writer(self):
         '''Create the File Writer.
 
@@ -760,7 +805,7 @@ class Builder(object):
         args = self._args
 
         if args.delete_after or args.output_document:
-            return NullWriter()
+            return self._factory.new('FileWriter')  # is a NullWriter
 
         use_dir = (len(args.urls) != 1 or args.page_requisites
                    or args.recursive)
@@ -807,7 +852,10 @@ class Builder(object):
         else:
             file_class = AntiClobberFileWriter
 
-        return file_class(
+        self._factory.class_map['FileWriter'] = file_class
+
+        return self._factory.new(
+            'FileWriter',
             path_namer,
             file_continuing=args.continue_download,
             headers_included=args.save_headers,
